/*************************************************
*                                                *
* SiTCP Bus Control Protocol                     *
* Test program                                   *
*                                                *
* 2006/03/01 Tomohisa Uchida                     *
*                                                *
*************************************************/

#include "rbcp.h"

int main(int argc, char* argv[]){

  struct timeval timeout;
  fd_set setSelect;

  /* Check inut values */
  if(argc != 3){
    puts("\nThis application controls bus of a SiTCP chip for debugging.");
    printf("Usage: %s <IP address> <Port #>\n\n", argv[0]);
    
    exit(EXIT_SUCCESS);
    }
  
  char* sitcpIpAddr;
  unsigned int sitcpPort;
  
  
  /* Get IP address and port # of a SiTCP */
  sitcpIpAddr = argv[1];
  sitcpPort   = atoi(argv[2]);
  
  
  /* Create a Socket */
  puts("\nCreate socket...\n");

  int sock;

  sock = socket(AF_INET, SOCK_DGRAM, 0);

  struct sockaddr_in sitcpAddr;
  
  sitcpAddr.sin_family      = AF_INET;
  sitcpAddr.sin_port        = htons(sitcpPort);
  sitcpAddr.sin_addr.s_addr = inet_addr(sitcpIpAddr);


  /* Send data */

  unsigned int tempKeyBuf;
  struct bcp_header sndHeader;

  /* Ver and type */
  printf("Ver./ Type (Hex) =");
  scanf("%x",&tempKeyBuf);
  if(tempKeyBuf>255){
    puts("\nError: Ver./Type field is from 0x00 through 0xFF\n");
  }
  sndHeader.type=(unsigned char)tempKeyBuf;

  /* Command / Flag */
  printf("Command / Flag (Hex) =");
  scanf("%x",&tempKeyBuf);
  if(tempKeyBuf>255){
    puts("\nError: Command/Flag field is from 0x00 through 0xFF\n");
  }

  sndHeader.command=(unsigned char)tempKeyBuf;

  /* ID */
  printf("ID # (Dec) =");
  scanf("%d",&tempKeyBuf);
  if(tempKeyBuf>255){
    puts("\nError: ID # is from 0 through 255\n");
  }

  sndHeader.id=(unsigned char)tempKeyBuf;

  /* Length */
  printf("Length (Dec) =");
  scanf("%d",&tempKeyBuf);
  if(tempKeyBuf>255){
    puts("\nError: length is from 0 through 255\n");
  }

  sndHeader.length=(unsigned char)tempKeyBuf;

  /* Address */
  printf("address (Hex) =");
  scanf("%x",&tempKeyBuf);
	
  sndHeader.address=htonl(tempKeyBuf);

  /* Data */
  unsigned char sndData[256];
  int tempData;
  int endFlag =0;
  int sndDataLen = 0;

  sndData[0]=0;

  while(sndDataLen<256 && endFlag==0){
    printf("data[%x] (Hex)=",sndDataLen+tempKeyBuf);
    if(scanf("%x",&tempData)!=1) endFlag=1;
    sndData[sndDataLen]=tempData;
    if(tempData<0 || tempData>255) endFlag=1;
    sndDataLen++;
  }

  sndDataLen--;
  
  /* Display Header */
  printf("\n");
  printf("\n");
  printf("\tVer/Type : 0x%.2x\n",sndHeader.type);
  printf("\tCmd/Flag : 0x%.2x\n",sndHeader.command);
  printf("\tID       : 0x%.2x\n",sndHeader.id);
  printf("\tLength   : 0x%.2x\n",sndHeader.length);
  printf("\tAddress  : 0x%.8x\n",ntohl(sndHeader.address));

  int i;

  for(i=0; i<sndDataLen; i++){
    printf("\tData[0x%x]: 0x%.2x\n",ntohl(sndHeader.address)+i,sndData[i]);    
  }
  printf("\n");

  printf("Send this packet to %s:%d\n",sitcpIpAddr,sitcpPort);

  char tempKeyStr;

  printf("y(Yes) or n(No) =");
  while(tempKeyStr!='y' && tempKeyStr!='Y' && 
	tempKeyStr!='n' && tempKeyStr!='N'){
    tempKeyStr=getchar();
  }

  if(tempKeyStr=='y' || tempKeyStr=='Y'){
    puts("\n Tring to sent the packet...\n");
  }else{
    puts("\nExit!");
    exit(1);
  }

  unsigned char sndBuf[2048];

  memcpy(sndBuf,&sndHeader, sizeof(sndHeader));
  memcpy(sndBuf+sizeof(sndHeader),sndData,sndDataLen);

  int j = 0;

  for(i=0; i< sndDataLen + sizeof(sndHeader);i++){
    if(j==0) {
      printf("\t[%.3x]:%.2x ",i,sndBuf[i]);
      j++;
    }else if(j==3){
      printf("%.2x\n",sndBuf[i]);
      j=0;
    }else{
      printf("%.2x ",sndBuf[i]);
      j++;
    }
  }
  puts("\n");
  
  sendto(sock, sndBuf, sndDataLen + sizeof(sndHeader), 0, (struct sockaddr *)&sitcpAddr, sizeof(sitcpAddr));

  puts("The packet have been sent!\n");

  /* Receive packets*/
  
  int rcvdBytes;
  unsigned char rcvdBuf[1024];

  puts("\nWait to receive the ACK packet...");

  int recvVal = 1;

  while(recvVal){

    FD_ZERO(&setSelect);
    FD_SET(sock, &setSelect);

    timeout.tv_sec  = 3;
    timeout.tv_usec = 0;
 
    if(select(sock+1, &setSelect, NULL, NULL,&timeout)==0){
      /* time out */
      recvVal = 0;
      puts("\n***** Timeout ! *****");

    } else {
      /* receive packet */
      if(FD_ISSET(sock,&setSelect)){
	rcvdBytes=recvfrom(sock, rcvdBuf, 2048, 0, NULL, NULL);
	rcvdBuf[rcvdBytes]=0;
	puts("\n***** A pacekt is received ! *****.");
	puts("Received data:");

	j=0;

	for(i=0; i<rcvdBytes; i++){
	  if(j==0) {
	    printf("\t[%.3x]:%.2x ",i,rcvdBuf[i]);
	    j++;
	  }else if(j==3){
	    printf("%.2x\n",rcvdBuf[i]);
	    j=0;
	  }else{
	    printf("%.2x ",rcvdBuf[i]);
	    j++;
	  }
	}
	puts("\n");

	recvVal = 0;
      }
    }
  }


  close(sock);
}
